window.Gleam.initSearch([{"doc":"funsies","title":"funsies","content":"# funsies\n\n&gt; Use your database with childlike enthusiasm. Its more than fun-- it&#39;s funsies!\n\nA fun, type-safe ORM for `Gleam`! (Work in progress, expect big changes! ALSO ASK QUESTIONS IF YOU WANT/NEED/ARE INTERESTED IN ANY OF IT!)\n\n&gt; [!IMPORTANT]\n&gt; Now with a CLI! Now, have your schemas automatically generate types and decoders!\n\nThese docs are a bit out of date. Will soon be updated.\n\n## About\n\n`Funsies` is a work in progress.\n\nMy goal with `funsies` was to create an easy-to-use, friendly ORM for this magical language we call `Gleam`.\n\nI think that Gleam has a ton of potential for the web (with `erlang` and `javascript` targets). But, to date the DB landscape in Gleam is wonderful, but a bit bare-bones.\n\nThere are a few great projects (like squirrel) for using raw SQL, and some libraries for generating queries. There were not, however, many libraries for real, full-featured ORMs.\n\nThat&#39;s what `funsies` is for!\n\n`Funsies` has awesome code-gen abilities, and type-safety, and is (in my opinion) fun to use.\n\n## Usage (with CLI)\n\n`Funsies` works based on Schemas. These schemas are used all the time when using `funsies`.\n\nIn the pre-CLI days, you would have to use the provided functions to generate the types and decoders.\n\nThis came with a couple of challenges:\n\n- Where do I keep my schemas? Are they separate from the rest of my code?\n- How do I create the types and decoders? Should I write a script? Just call a function? Or, worst of all, write it by hand?!\n\nNow, we have a CLI to manage the hard parts! All you do is create a `src/schema` folder. In that folder, you would create a file along the lines of `{schema_name}.gleam` and inside you would make a public function like this:\n\n```gleam\npub fn {schema_name} {\n{schema}\n}\n```\n\nWhere schema is the schema you would normally put in your code, and the schema name is the same thing that you put in your file name.\n\nNow, run `gleam run -m funsies` and it will magically generate a new folder (`src/funs`) with files for each and every one of your Schemas!\n\n## Usage\n\n`funsies` is designed to be used with the `gleam_pgo` package for connecting to a postgres database. Currently, it only supports `postgres` database. Eventually I plan for it to support many DBs.\n\nTo use funsies, clone the repo, add it to your project, and read through the tutorial below.\n\n## Tutorial\n\nFirst, set up a database object with the `gleam_pgo` package.\n\n```gleam\n  let db =\n    pgo.connect(\n      pgo.Config(\n        ..pgo.default_config(),\n        host: &quot;localhost&quot;,\n        database: &quot;your_db_name&quot;,\n        user: &quot;db_username&quot;,\n        port: 5432,\n        password: option.Some(&quot;db_password&quot;),\n        pool_size: 15,\n      ),\n    )\n```\n\nNow, the fun begins!\n\nWhen using `funsies`, you will be working with `Tables`. Tables are made using the `schema` module, which provides a DSL for building gleam structs that fully represent the table in the database, and are used all over the ORM.\n\nTo create a table, use syntax like the following:\n\n```gleam\nlet user_table =\nschema.create_table(&quot;table_name&quot;)\n|&gt; schema.add_serial_column(&quot;id&quot;) /// This is a column that autoincrements on insert.\n|&gt; schema.add_string_column(&quot;name&quot;, 255) // this represents text\n|&gt; schema.add_int_column(&quot;age&quot;) // this represents an integer\n|&gt; schema.add_bool_column(&quot;is_active&quot;) //this represents a boolean!\n```\n\nThis table object is POWERFUL. It is used to generate types, to perform queries, etc.\n\nTo use this table, we can first perform a migration to create the table in our database. We need to generate the SQL, and then run it (this part I eventually want to make more seamless. Add a PR if you want to help!)\n\n```gleam\nlet user_table_sql = schema.generate_create_table_sql(user_table)\nuser_table_sql\n|&gt; pgo.execute(db, [], dynamic.dynamic)\n```\n\nNow, we have a table in our database!\n\nThe next step is to generate some types and decoders. This, as mentioned earlier, uses the table value!\n\nDecoders and types are generated like this:\n\n```gleam\nimport db/decoder\ndecoder.generate_row_type(user_table)\ndecoder.generate_decoder_code(user_table)\n```\n\nThen the magic happens! Automatically, you will have a type and decoder generated for you! They will be in the `src/schema` directory, under files named after the table. The type will be named `{table_name}Row{columns}`, and the decoder will be named `{table_name}_decoder_{columns}`. (ALSO WORKING ON THIS TO MAKE NICER NAMES)\n\nNow, we can import our generated types and use them to create an `ORM`.\n\n```gleam\nimport db/orm\nimport schema/users6\nimport schema/users6_decoder\n\nlet user_decoder = users6_decoder.users6_decoder_idnameisactive()\n\nlet user_orm =\n  orm.orm(users6.Users6RowIdNameIsactive, user_table, db, user_decoder) // this is the ORM! The IdNameIsactive is the columns of the table we made earlier. This is so we can have many different types/decoders if we want to select only some columns!\n```\n\nNow the ORM can be used to perform queries!\n\n```gleam\nlet fetched_user_result = user_orm.by_id(2)\n\nio.debug(fetched_user_result)  // This will be serialized into the types you generated earlier! No more messing around with `dynamic` types!\n\nlet all_users_result = user_orm.get_all()\n\nio.debug(all_users_result)\n\nlet new_user_values = [pgo.text(&quot;Asher&quot;), pgo.int(15), pgo.bool(True)] // no ID since auto-increments!\n\nlet insert_result = user_orm.insert(new_user_values)\n\nio.debug(insert_result)\n```\n\nMagical, right?\n\nFinally, we can use the `yum` DSL to build more complex queries.\n\n```gleam\nimport db/query/yum\n\nlet query =\n  yum.new(&quot;users6&quot;)\n  |&gt; yum.select([&quot;name&quot;, &quot;is_active&quot;])\n  |&gt; yum.where(&quot;is_active = TRUE&quot;)\n  |&gt; yum.to_sql()\n```\n\nThis will return a string SQL query. To use it, you can generate new types using only the columns you are returning and have it decode into that decoder! Or just use the dynamic type if you want :)\n\n## Contributing\n\nPLEASE CONTRIBUTE! I am new to Gleam, and if there are any improvements, docs, tests, new features, improvements of current features, or anything else you want to contribute, PLEASE make a PR! I would LOVE the help.\n\nTHANKS SO MUCH!\n","url":"index.html"},{"doc":"cli/project","title":"create_files","content":"pub fn create_files(\n  input: Dict(String, String),\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\ncli/project.create_files\ncli/project create_files","url":"cli/project.html#create_files"},{"doc":"cli/project","title":"find_schema_files","content":"pub fn find_schema_files() -&gt; Result(Dict(String, String), Nil)\n\nSynonyms:\ncli/project.find_schema_files\ncli/project find_schema_files","url":"cli/project.html#find_schema_files"},{"doc":"cli/project","title":"name","content":"pub fn name() -&gt; Result(String, Nil)\n Returns the project&#39;s name, read from the `gleam.toml` file.\n\nSynonyms:\ncli/project.name\ncli/project name","url":"cli/project.html#name"},{"doc":"cli/project","title":"src","content":"pub fn src() -&gt; String\n Finds the path of the project&#39;s `src` directory.\n This recursively walks up from the current directory until it finds a\n `gleam.toml` and builds it from there.\n\nSynonyms:\ncli/project.src\ncli/project src","url":"cli/project.html#src"},{"doc":"cli/project","title":"work","content":"pub fn work() -&gt; Result(Nil, Nil)\n\nSynonyms:\ncli/project.work\ncli/project work","url":"cli/project.html#work"},{"doc":"cli/project","title":"cli/project","content":" A set of utilities to work within a Gleam project. I TOOK MOST OF THIS FROM THE SQUIRREL REPO. TYSM!\n","url":"cli/project.html"},{"doc":"db/schema","title":"Column","content":"pub type Column {\n  StringColumn(name: String, size: Int)\n  IntColumn(name: String)\n  BoolColumn(name: String)\n  ForeignKeyColumn(\n    name: String,\n    references_table: String,\n    references_column: String,\n    references_type: String,\n  )\n  SerialColumn(name: String)\n}\n\nStringColumn(name: String, size: Int)\n\n\nIntColumn(name: String)\n\n\nBoolColumn(name: String)\n\n\nForeignKeyColumn(\n  name: String,\n  references_table: String,\n  references_column: String,\n  references_type: String,\n)\n\n\nSerialColumn(name: String)\n\n\nSynonyms:\ndb/schema.Column\ndb/schema Column","url":"db/schema.html#Column"},{"doc":"db/schema","title":"Table","content":"pub type Table {\n  Table(name: String, columns: List(Column))\n}\n\nTable(name: String, columns: List(Column))\n\n\nSynonyms:\ndb/schema.Table\ndb/schema Table","url":"db/schema.html#Table"},{"doc":"db/schema","title":"add_bool_column","content":"pub fn add_bool_column(table: Table, name: String) -&gt; Table\n\nSynonyms:\ndb/schema.add_bool_column\ndb/schema add_bool_column","url":"db/schema.html#add_bool_column"},{"doc":"db/schema","title":"add_foreign_key_column","content":"pub fn add_foreign_key_column(\n  table: Table,\n  name: String,\n  references_table: Table,\n  references_column: String,\n) -&gt; Table\n\nSynonyms:\ndb/schema.add_foreign_key_column\ndb/schema add_foreign_key_column","url":"db/schema.html#add_foreign_key_column"},{"doc":"db/schema","title":"add_int_column","content":"pub fn add_int_column(table: Table, name: String) -&gt; Table\n\nSynonyms:\ndb/schema.add_int_column\ndb/schema add_int_column","url":"db/schema.html#add_int_column"},{"doc":"db/schema","title":"add_serial_column","content":"pub fn add_serial_column(table: Table, name: String) -&gt; Table\n\nSynonyms:\ndb/schema.add_serial_column\ndb/schema add_serial_column","url":"db/schema.html#add_serial_column"},{"doc":"db/schema","title":"add_string_column","content":"pub fn add_string_column(\n  table: Table,\n  name: String,\n  size: Int,\n) -&gt; Table\n\nSynonyms:\ndb/schema.add_string_column\ndb/schema add_string_column","url":"db/schema.html#add_string_column"},{"doc":"db/schema","title":"create_table","content":"pub fn create_table(name: String) -&gt; Table\n\nSynonyms:\ndb/schema.create_table\ndb/schema create_table","url":"db/schema.html#create_table"},{"doc":"db/schema","title":"generate_create_table_sql","content":"pub fn generate_create_table_sql(table: Table) -&gt; String\n\nSynonyms:\ndb/schema.generate_create_table_sql\ndb/schema generate_create_table_sql","url":"db/schema.html#generate_create_table_sql"},{"doc":"db/schema","title":"generate_drop_table_sql","content":"pub fn generate_drop_table_sql(table: Table) -&gt; String\n\nSynonyms:\ndb/schema.generate_drop_table_sql\ndb/schema generate_drop_table_sql","url":"db/schema.html#generate_drop_table_sql"},{"doc":"db/schema","title":"get_part_table","content":"pub fn get_part_table(\n  table: Table,\n  columns: List(String),\n) -&gt; Table\n\nSynonyms:\ndb/schema.get_part_table\ndb/schema get_part_table","url":"db/schema.html#get_part_table"},{"doc":"db/schema","title":"db/schema","content":"","url":"db/schema.html"},{"doc":"db/decoder","title":"generate_decoder_code","content":"pub fn generate_decoder_code(table: Table) -&gt; String\n\nSynonyms:\ndb/decoder.generate_decoder_code\ndb/decoder generate_decoder_code","url":"db/decoder.html#generate_decoder_code"},{"doc":"db/decoder","title":"generate_row_type","content":"pub fn generate_row_type(table: Table) -&gt; String\n\nSynonyms:\ndb/decoder.generate_row_type\ndb/decoder generate_row_type","url":"db/decoder.html#generate_row_type"},{"doc":"db/decoder","title":"db/decoder","content":"","url":"db/decoder.html"},{"doc":"templates/by_id","title":"render","content":"pub fn render(table table: Table) -&gt; String\n\nSynonyms:\ntemplates/by_id.render\ntemplates/by_id render","url":"templates/by_id.html#render"},{"doc":"templates/by_id","title":"render_builder","content":"pub fn render_builder(table table: Table) -&gt; StringBuilder\n\nSynonyms:\ntemplates/by_id.render_builder\ntemplates/by_id render_builder","url":"templates/by_id.html#render_builder"},{"doc":"templates/by_id","title":"templates/by_id","content":"","url":"templates/by_id.html"},{"doc":"templates/get_all","title":"render","content":"pub fn render(table table: Table) -&gt; String\n\nSynonyms:\ntemplates/get_all.render\ntemplates/get_all render","url":"templates/get_all.html#render"},{"doc":"templates/get_all","title":"render_builder","content":"pub fn render_builder(table table: Table) -&gt; StringBuilder\n\nSynonyms:\ntemplates/get_all.render_builder\ntemplates/get_all render_builder","url":"templates/get_all.html#render_builder"},{"doc":"templates/get_all","title":"templates/get_all","content":"","url":"templates/get_all.html"},{"doc":"templates/insert","title":"render","content":"pub fn render(table table: Table) -&gt; String\n\nSynonyms:\ntemplates/insert.render\ntemplates/insert render","url":"templates/insert.html#render"},{"doc":"templates/insert","title":"render_builder","content":"pub fn render_builder(table table: Table) -&gt; StringBuilder\n\nSynonyms:\ntemplates/insert.render_builder\ntemplates/insert render_builder","url":"templates/insert.html#render_builder"},{"doc":"templates/insert","title":"templates/insert","content":"","url":"templates/insert.html"},{"doc":"db/orm_gen","title":"create_by_id","content":"pub fn create_by_id(\n  path: String,\n  table: Table,\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\ndb/orm_gen.create_by_id\ndb/orm_gen create_by_id","url":"db/orm_gen.html#create_by_id"},{"doc":"db/orm_gen","title":"create_get_all","content":"pub fn create_get_all(\n  path: String,\n  table: Table,\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\ndb/orm_gen.create_get_all\ndb/orm_gen create_get_all","url":"db/orm_gen.html#create_get_all"},{"doc":"db/orm_gen","title":"create_insert","content":"pub fn create_insert(\n  path: String,\n  table: Table,\n) -&gt; Result(Nil, FileError)\n\nSynonyms:\ndb/orm_gen.create_insert\ndb/orm_gen create_insert","url":"db/orm_gen.html#create_insert"},{"doc":"db/orm_gen","title":"db/orm_gen","content":"","url":"db/orm_gen.html"},{"doc":"db/query/yum","title":"Eq","content":"pub type Eq {\n  Equals(col: String, value: Value)\n}\n\nEquals(col: String, value: Value)\n\n\nSynonyms:\ndb/query/yum.Eq\ndb/query/yum Eq","url":"db/query/yum.html#Eq"},{"doc":"db/query/yum","title":"QueryBuilder","content":"pub type QueryBuilder {\n  QueryBuilder(\n    table: schema.Table,\n    select_columns: List(String),\n    where_clauses: List(String),\n    order_by_clauses: List(String),\n  )\n}\n\nQueryBuilder(\n  table: schema.Table,\n  select_columns: List(String),\n  where_clauses: List(String),\n  order_by_clauses: List(String),\n)\n\n\nSynonyms:\ndb/query/yum.QueryBuilder\ndb/query/yum QueryBuilder","url":"db/query/yum.html#QueryBuilder"},{"doc":"db/query/yum","title":"Value","content":"pub type Value {\n  IntValue(Int)\n  StringValue(String)\n  BoolValue(Bool)\n  ErrorValue(String)\n}\n\nIntValue(Int)\n\n\nStringValue(String)\n\n\nBoolValue(Bool)\n\n\nErrorValue(String)\n\n\nSynonyms:\ndb/query/yum.Value\ndb/query/yum Value","url":"db/query/yum.html#Value"},{"doc":"db/query/yum","title":"WhereBuilder","content":"pub type WhereBuilder {\n  WhereBuilder(table: schema.Table, clauses: List(String))\n}\n\nWhereBuilder(table: schema.Table, clauses: List(String))\n\n\nSynonyms:\ndb/query/yum.WhereBuilder\ndb/query/yum WhereBuilder","url":"db/query/yum.html#WhereBuilder"},{"doc":"db/query/yum","title":"equals","content":"pub fn equals(\n  builder: WhereBuilder,\n  col: String,\n  value: a,\n) -&gt; WhereBuilder\n\nSynonyms:\ndb/query/yum.equals\ndb/query/yum equals","url":"db/query/yum.html#equals"},{"doc":"db/query/yum","title":"ere","content":"pub fn ere(table: Table) -&gt; WhereBuilder\n\nSynonyms:\ndb/query/yum.ere\ndb/query/yum ere","url":"db/query/yum.html#ere"},{"doc":"db/query/yum","title":"new","content":"pub fn new(table: Table) -&gt; QueryBuilder\n\nSynonyms:\ndb/query/yum.new\ndb/query/yum new","url":"db/query/yum.html#new"},{"doc":"db/query/yum","title":"order_by","content":"pub fn order_by(\n  builder: QueryBuilder,\n  column: String,\n  direction: String,\n) -&gt; QueryBuilder\n\nSynonyms:\ndb/query/yum.order_by\ndb/query/yum order_by","url":"db/query/yum.html#order_by"},{"doc":"db/query/yum","title":"select","content":"pub fn select(builder: QueryBuilder) -&gt; QueryBuilder\n\nSynonyms:\ndb/query/yum.select\ndb/query/yum select","url":"db/query/yum.html#select"},{"doc":"db/query/yum","title":"to_sql","content":"pub fn to_sql(builder: QueryBuilder) -&gt; String\n\nSynonyms:\ndb/query/yum.to_sql\ndb/query/yum to_sql","url":"db/query/yum.html#to_sql"},{"doc":"db/query/yum","title":"to_where","content":"pub fn to_where(builder: WhereBuilder) -&gt; List(String)\n\nSynonyms:\ndb/query/yum.to_where\ndb/query/yum to_where","url":"db/query/yum.html#to_where"},{"doc":"db/query/yum","title":"wh","content":"pub fn wh(\n  builder: QueryBuilder,\n  condition: List(String),\n) -&gt; QueryBuilder\n\nSynonyms:\ndb/query/yum.wh\ndb/query/yum wh","url":"db/query/yum.html#wh"},{"doc":"db/query/yum","title":"db/query/yum","content":"","url":"db/query/yum.html"},{"doc":"db/query/yummy","title":"Eq","content":"pub type Eq {\n  Equals(col: String, value: Value)\n  Not(Eq)\n  Or(Eq, Eq)\n  And(Eq, Eq)\n}\n\nEquals(col: String, value: Value)\n\n\nNot(Eq)\n\n\nOr(Eq, Eq)\n\n\nAnd(Eq, Eq)\n\n\nSynonyms:\ndb/query/yummy.Eq\ndb/query/yummy Eq","url":"db/query/yummy.html#Eq"},{"doc":"db/query/yummy","title":"QueryBuilder","content":"pub type QueryBuilder {\n  QueryBuilder(\n    table: schema.Table,\n    select_columns: List(String),\n    where_clauses: List(Eq),\n    order_by_clauses: List(String),\n  )\n}\n\nQueryBuilder(\n  table: schema.Table,\n  select_columns: List(String),\n  where_clauses: List(Eq),\n  order_by_clauses: List(String),\n)\n\n\nSynonyms:\ndb/query/yummy.QueryBuilder\ndb/query/yummy QueryBuilder","url":"db/query/yummy.html#QueryBuilder"},{"doc":"db/query/yummy","title":"Value","content":"pub type Value {\n  IntValue(Int)\n  StringValue(String)\n  BoolValue(Bool)\n  ErrorValue(String)\n}\n\nIntValue(Int)\n\n\nStringValue(String)\n\n\nBoolValue(Bool)\n\n\nErrorValue(String)\n\n\nSynonyms:\ndb/query/yummy.Value\ndb/query/yummy Value","url":"db/query/yummy.html#Value"},{"doc":"db/query/yummy","title":"WhereBuilder","content":"pub type WhereBuilder {\n  WhereBuilder(table: schema.Table, clauses: List(Eq))\n}\n\nWhereBuilder(table: schema.Table, clauses: List(Eq))\n\n\nSynonyms:\ndb/query/yummy.WhereBuilder\ndb/query/yummy WhereBuilder","url":"db/query/yummy.html#WhereBuilder"},{"doc":"db/query/yummy","title":"and","content":"pub fn and(builder: WhereBuilder) -&gt; WhereBuilder\n\nSynonyms:\ndb/query/yummy.and\ndb/query/yummy and","url":"db/query/yummy.html#and"},{"doc":"db/query/yummy","title":"equals","content":"pub fn equals(\n  builder: WhereBuilder,\n  col: String,\n  value: a,\n) -&gt; WhereBuilder\n\nSynonyms:\ndb/query/yummy.equals\ndb/query/yummy equals","url":"db/query/yummy.html#equals"},{"doc":"db/query/yummy","title":"new","content":"pub fn new(table: Table) -&gt; QueryBuilder\n\nSynonyms:\ndb/query/yummy.new\ndb/query/yummy new","url":"db/query/yummy.html#new"},{"doc":"db/query/yummy","title":"not","content":"pub fn not(builder: WhereBuilder) -&gt; WhereBuilder\n\nSynonyms:\ndb/query/yummy.not\ndb/query/yummy not","url":"db/query/yummy.html#not"},{"doc":"db/query/yummy","title":"or","content":"pub fn or(builder: WhereBuilder) -&gt; WhereBuilder\n\nSynonyms:\ndb/query/yummy.or\ndb/query/yummy or","url":"db/query/yummy.html#or"},{"doc":"db/query/yummy","title":"order_by","content":"pub fn order_by(\n  builder: QueryBuilder,\n  column: String,\n  direction: String,\n) -&gt; QueryBuilder\n\nSynonyms:\ndb/query/yummy.order_by\ndb/query/yummy order_by","url":"db/query/yummy.html#order_by"},{"doc":"db/query/yummy","title":"select","content":"pub fn select(builder: QueryBuilder) -&gt; QueryBuilder\n\nSynonyms:\ndb/query/yummy.select\ndb/query/yummy select","url":"db/query/yummy.html#select"},{"doc":"db/query/yummy","title":"to_sql","content":"pub fn to_sql(builder: QueryBuilder) -&gt; Result(String, String)\n\nSynonyms:\ndb/query/yummy.to_sql\ndb/query/yummy to_sql","url":"db/query/yummy.html#to_sql"},{"doc":"db/query/yummy","title":"wb","content":"pub fn wb(table: Table) -&gt; WhereBuilder\n\nSynonyms:\ndb/query/yummy.wb\ndb/query/yummy wb","url":"db/query/yummy.html#wb"},{"doc":"db/query/yummy","title":"where","content":"pub fn where(\n  builder: QueryBuilder,\n  condition: WhereBuilder,\n) -&gt; QueryBuilder\n\nSynonyms:\ndb/query/yummy.where\ndb/query/yummy where","url":"db/query/yummy.html#where"},{"doc":"db/query/yummy","title":"db/query/yummy","content":"","url":"db/query/yummy.html"},{"doc":"funsies","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\nfunsies.main\nfunsies main","url":"funsies.html#main"},{"doc":"funsies","title":"funsies","content":"","url":"funsies.html"}]);